// Package
package org.auxilum.opengl.common;

// Standard
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.Map;

// Custom
import org.auxilum.opengl.robot.SceneRenderer;


// Frame to display config options and info
public class SystemConfigFrame extends JDialog implements ChangeListener, ActionListener
{
	// Attributes
	private JButton okButton = null;
	private JLabel fpsLabel = null;
	private JSlider fpsSlider = null;
	private SceneRenderer renderer = null;
	
	public SystemConfigFrame(Map rendererInfo, SceneRenderer renderer)
	{
		this.renderer = renderer;
		
		setSize(450, 300);
		setTitle("Config");
   		setResizable(false);
   		setModal(false);
   		Container c = getContentPane();
   		c.setLayout(new BorderLayout());
   	
   		okButton = new JButton("OK");
        okButton.addActionListener(this);
        
        JPanel panelControl = new JPanel(new GridLayout());

        JTabbedPane tabbedPane = new JTabbedPane();	        
        
        // Info panel
        JPanel panelInfo = new JPanel(new BorderLayout());
        JPanel panelInfoGL = new JPanel(new GridLayout(9,2));
        panelInfo.add(panelInfoGL, BorderLayout.NORTH);
        
       	panelInfoGL.add(new JLabel());
        panelInfoGL.add(new JLabel());
        
        panelInfoGL.add(new JLabel("  GFX vendor: "));
        panelInfoGL.add(new JLabel((String)rendererInfo.get("GL_VENDOR")));
        panelInfoGL.add(new JLabel("  GFX renderer: "));
        panelInfoGL.add(new JLabel((String)rendererInfo.get("GL_RENDERER")));
        panelInfoGL.add(new JLabel("  OpenGL version: "));
        panelInfoGL.add(new JLabel((String)rendererInfo.get("GL_VERSION")));
        
        panelInfoGL.add(new JLabel());
        panelInfoGL.add(new JLabel());
        
        panelInfoGL.add(new JLabel("  Java version: "));
        panelInfoGL.add(new JLabel(System.getProperty("java.version")));
        panelInfoGL.add(new JLabel("  Operating system: "));
        panelInfoGL.add(new JLabel(System.getProperty("os.name")));
        
       	panelInfoGL.add(new JLabel());
        panelInfoGL.add(new JLabel());
        
        fpsSlider = new JSlider(25, 103);
        fpsSlider.setValue(renderer.getMaxFPS() == 0 ? 103 : renderer.getMaxFPS());
        fpsSlider.addChangeListener(this);
        fpsLabel = new JLabel(renderer.getMaxFPS() > 0 ? "  Max FPS:  " + renderer.getMaxFPS() + " " : "  Max FPS:  Unlimited ");

       	panelInfoGL.add(fpsLabel);
        panelInfoGL.add(fpsSlider);
        
        tabbedPane.addTab("System", null, panelInfo, "System information");
        tabbedPane.setMnemonicAt(0, KeyEvent.VK_1);

		// About panel
        JPanel panelAbout = new JPanel(new BorderLayout());
        JTextArea text = new JTextArea(	"\n  This program is an attempt to simulate an ABB\n  industrial robotic arm.\n\n" + 
        								"  This program uses JOGL for OpenGL rendering.\n\n" + 
        								"  Written by Morten Haraldsen, spring 2004.\n" + 
        								"  E-mail: morten at auxilum dot com"
        							  );
		text.setFont(new Font("arial", Font.BOLD, 12));
        text.setEditable(false);
        text.setLineWrap(true);
        text.setWrapStyleWord(true);
        text.setBackground(new Color(224,224,224));
        panelAbout.add(text, BorderLayout.CENTER);
        							  
        tabbedPane.addTab("About", null, panelAbout,"About this program");
        tabbedPane.setMnemonicAt(1, KeyEvent.VK_2);

        //Add the tabbed pane to this panel.
        c.add(tabbedPane, BorderLayout.CENTER);
        c.add(okButton, BorderLayout.SOUTH);
	}
	
	
	/** */
	public void stateChanged(ChangeEvent e)
	{
		if (e.getSource() == fpsSlider)
		{
			int value = fpsSlider.getValue();

			if (value > 100)
			{
				renderer.setTargetFPS(0);
				fpsLabel.setText("  Max FPS:  Unlimited");
			}
			else
			{
				renderer.setTargetFPS(fpsSlider.getValue());
				fpsLabel.setText("  Max FPS:  " + value);
			}
		}
	} 
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okButton)
		{
			hide();	
		}
	}      
}  	