// Package
package org.auxilum.opengl.robot;

// Standard
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


/**
 *
 **/
public class RobotControlPanel extends JPanel implements ChangeListener, ActionListener
{
	private JSlider[] sliders = null;
	private JLabel[] labels = null;
	private ChangeListener changeListener = null;
	
	private JButton btnStart = null;
	private JButton btnPause = null;
	private JButton btnStop = null;
	private JButton btnReset = null;
	
	private JComboBox cmbPrograms = null;
	
	private RobotProgramExecutor programExecutor = null;
	private RobotStatus robotStatus = null;
	//private RobotProgram robotProgram = null;
	
	private Color foregroundColor = new Color(93, 110, 235);
	private Color backgroundColor = new Color(42, 42, 42);
	private Font font = new Font("courier", Font.PLAIN, 12);
	
	
	/**
	 * Constructor
	 **/
	public RobotControlPanel(RobotStatus status) 
	{
		this.robotStatus = status;
		
		Axis[] axises = status.getAxises();
		
		// Init sliders
		this.sliders = new JSlider[axises.length];
		
		// Init labels
		this.labels = new JLabel[axises.length];
		
		// Prepare layout manager
		JPanel mainPanel = new JPanel(new BorderLayout(0, 10));
		
		
		JPanel axisPanel = new JPanel(new GridLayout(2, (axises.length * 1), 20, 0));
		axisPanel.setBackground(backgroundColor);
		mainPanel.setBackground(backgroundColor);
		setBackground(backgroundColor);
		
		// Create sliders and labels
		for (int i = 0; i < axises.length; i++)
		{
			labels[i] = new JLabel("  " + axises[i].getName() + ":");
			labels[i].setForeground(new Color(93, 110, 235));
			labels[i].setBackground(backgroundColor);
			labels[i].setOpaque(true);
			labels[i].setFont(font);
			
			sliders[i] = new JSlider((int)axises[i].getMinAngle(), (int)axises[i].getMaxAngle());
			sliders[i].setBackground(backgroundColor);
			sliders[i].setForeground(foregroundColor);
			sliders[i].setPaintTicks(true);
			sliders[i].setMinorTickSpacing((int)(axises[i].getMaxAngle() + Math.abs(axises[i].getMinAngle())) / 10);
			sliders[i].setValue((int)axises[i].getCurrentAngle());
			sliders[i].setFont(font);
			axisPanel.add(labels[i]);
		}

		// Add sliders
		for (int i = 0; i < axises.length; i++)
		{
			axisPanel.add(sliders[i]);
		}
		
		// Add control buttons
		btnStart = new JButton("Start");
		btnStart.setBackground(backgroundColor);
		btnStart.setForeground(foregroundColor);
		btnStart.setFont(font);
		
		btnPause = new JButton("Pause");
		btnPause.setBackground(backgroundColor);
		btnPause.setForeground(foregroundColor);
		btnPause.setFont(font);
		
		btnStop = new JButton("Stop");
		btnStop.setBackground(backgroundColor);
		btnStop.setForeground(foregroundColor);
		btnStop.setFont(font);
		
		btnReset = new JButton("Reset");
		btnReset.setBackground(backgroundColor);
		btnReset.setForeground(foregroundColor);
		btnReset.setFont(font);
		
		JPanel buttonPanel = new JPanel(new BorderLayout(0,0));
		JPanel buttons = new JPanel(new GridLayout(1, 3));
		buttons.setBackground(backgroundColor);
		buttons.add(btnStart);
		buttons.add(btnPause);
		//buttons.add(btnStop);
		buttons.add(btnReset);
		
		// Disable buttons
		btnPause.setEnabled(false);
		btnStop.setEnabled(false);

		cmbPrograms = new JComboBox(RobotProgramList.getList("data/programs"));		
		cmbPrograms.setFont(font);
		
		// The program selector	
		cmbPrograms.setBackground(new Color(60,60,60));	
		cmbPrograms.setForeground(foregroundColor);
		
		// The buttons
		buttonPanel.setBackground(backgroundColor);
		buttonPanel.add(buttons, BorderLayout.CENTER);
		buttonPanel.add(cmbPrograms, BorderLayout.WEST);
		
		// Add buttons to main panel
		mainPanel.add(axisPanel, BorderLayout.CENTER);
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		// Add main panel to this panel
		add(mainPanel);
		

		// Add ChangeListener		
		for (int i = 0; i < sliders.length; i++)
		{
			sliders[i].addChangeListener(this);	
		}
		
		// Add ActionListener
		btnStart.addActionListener(this);
		btnPause.addActionListener(this);
		btnStop.addActionListener(this);
		btnReset.addActionListener(this);
		setBorder(new BevelBorder(BevelBorder.RAISED));
		
		// Init program executor
		programExecutor = new RobotProgramExecutor(robotStatus, this);
		
	}	
	
	/**
   	 *
   	 **/
   	public void stateChanged(ChangeEvent e)
   	{
   		int axisId = getComponentId(e.getSource());
   		robotStatus.setAxisAngle(axisId, ((JSlider)e.getSource()).getValue());	
   	}
   	
   	
   	   	
   	/**
   	 *
   	 **/
   	public void actionPerformed(ActionEvent e)
   	{
		if (e.getActionCommand().equals("Start"))
   		{
			start();
   		}
   		else if (e.getActionCommand().equals("Pause"))
   		{
   			pause();
   		}
   		else if (e.getActionCommand().equals("Stop"))
   		{
   			stop();
   		} 
   		else if (e.getActionCommand().equals("Reset"))
   		{
   			for (int i = 0; i < robotStatus.getAxises().length; i++)
   			{
   				robotStatus.setAxisAngle(i, 0.0f);
   			}
   		}    		
   	}
   	
   	
   	/**
   	 *
   	 **/
   	private int getComponentId(Object component)
	{
		for(int i = 0; i < sliders.length; i++)
		{
			if (component == sliders[i])
			{
				return i;
			}
		}
		
		return -1;
	}
	
	
	/**
	 *
	 **/
	public void start()
	{
		setAutoMode(true);
		btnReset.setEnabled(false);
		btnStart.setEnabled(false);
		btnPause.setEnabled(true);
		btnStop.setEnabled(true);
		
		RobotProgram robotProgram = new RobotProgram("data/programs/" + (String)cmbPrograms.getSelectedItem());
		programExecutor.startRobotProgram(robotProgram.getMovements());
	}
	
	/**
	 *
	 **/
	public void pause()
	{
		programExecutor.pauseRobotProgram();
		btnStart.setEnabled(true);
		btnPause.setEnabled(false);
	}
	
	/**
	 *
	 */
	public void stop()
	{
		programExecutor.stopRobotProgram();
		btnStart.setEnabled(true);
		btnReset.setEnabled(true);
		btnPause.setEnabled(false);
		btnStop.setEnabled(false);
		setAutoMode(false);
		
	}
	
	
	/**
	 *
	 **/
	private void setAutoMode(boolean auto)
	{
		for(int i = 0; i < sliders.length; i++)
		{
			sliders[i].setEnabled(!auto);
			labels[i].setEnabled(!auto);
			
			if (auto)
			{
				sliders[i].removeChangeListener(this);
			}
			else
			{
				sliders[i].addChangeListener(this);
			}
		}
	}
	
	
	/**
	 *
	 **/
	void setAxisValue(int axisId, int value)
	{
		this.sliders[axisId].setValue(value);
	}
	
	
	/**
	 *
	 **/
	public void programFinished()
	{
		if (programExecutor != null)
			stop();
	}
}
